#
#               Common Quantification
#                        of
#              Categorical Variables
#
#             Yasuharu Okamoto, 2016.09, 2023.05
#
import pandas as pd
import math
import numpy as np
import scipy.stats as ss

in_fl_nm = input('データファイル(*.xlsx) = ')
with pd.ExcelFile(in_fl_nm) as f:
    data = pd.read_excel(f)

f_out = open('Results.txt', 'w')
f_out.write('Data file = '+ in_fl_nm)

print(data)
var_names = [v for v in data.keys()]
print('var_names...\n', var_names)
var_cond = data.values[0]             #    変換の種類
print('var_cond...\n', var_cond)
print('data.values...\n', data.values)
X = data.values[1:]                   #     ２行目以降を取り出す
print('X =\n', X)
n_data, n_vars = np.shape(X)
print('n_data =', n_data, '   n_vars =', n_vars)

dict_v = dict()
for i_var in range(0, n_vars):
    if (var_cond[i_var] < 0):               #  不適当な値
        print("var_cond < 0")
        raise "var_cond error."
    else:        
        if (var_cond[i_var] == 0):          #  無変換
            dict_v[var_names[i_var]] = X.T[i_var]
        elif (var_cond[i_var] == 1):        #  標準化
            vX = ss.zscore(ToFloat(X.T[i_var]))
            dict_v[var_names[i_var]] = vX
        else:                               #  数量化
            vX = X.T[i_var]
            K = var_cond[i_var]
            G = np.zeros((n_data, K))
            for j in range(n_data):
                G[j][int(vX[j])-1] = 1.0
            f = np.ones((1,n_data)) @ G / n_data
            H = G - np.ones((n_data, 1)) @ f
            Lmbd, Wght = np.linalg.eigh(H.T @ H)     #   拙著式(8.1.3)
            idx = np.argsort(-Lmbd)    #  大きいものから小さいものへの並べ換え用インデックス
            Lmbd = Lmbd[idx][:-1]
            Wght = Wght.T[idx][:-1].T
            coeff = (Lmbd ** (-0.5)) * (n_data**0.5)
            z = H @ Wght @ np.diag(coeff)            #   拙著式(8.1.4)
            for i in range(len(Lmbd)):
                dict_v[f'{var_names[i_var]}-{i}'] = z.T[i]
                f_out.write(f'\n{var_names[i_var]}-{i}:\n')
                f_out.write(f'    Weight =\n')
                for j in range(K):
                    f_out.write(f'           {Wght.T[i][j]:.5f}\n')
                         
#       
#      変換データのExcelファイルへの書き出し
#
with pd.ExcelWriter('Results.xlsx') as f:
    pd.DataFrame(dict_v).to_excel(f, index=False)
print('\nResults.xlsx was saved.')
          
f_out.close()
print('Results.txt was saved.')

